require File.dirname(__FILE__) + '/../spec_helper'
require 'appengine-utils/model/relationship'
require 'dm-core'
require 'dm-validations'
#require 'dm-serializer'

DataMapper.setup(:default, "appengine://auto")
DataMapper.repository.adapter.singular_naming_convention!
      

describe DataMapper::Model::Relationship do

  it 'should create a DM Model with a blob attribute' do
    model = FixtureModel.new
    model.methods.include?("test_blob").should be_true
  end
  
  it 'should create a DM Model with a blob relation property' do
    model = FixtureModel.new
    model.methods.include?("test_blob_id").should be_true
    model.methods.include?("test_blob_id=").should be_true
  end
  
  describe "filter methods" do
    
    it 'should have a filter method for save' do
      model = FixtureModel.new
      model.methods.include?("before_save_test_blob").should be_true
    end
  
    it 'should have a filter method for destroy' do
      model = FixtureModel.new
      model.methods.include?("before_destroy_test_blob").should be_true
    end
  end
  
  describe "call the filter methods" do
    it 'should call the before save method' do
      model = FixtureModel.new(:name => "test model")
      model.should_receive(:before_save_test_blob).should be_true
      model.save 
    end
      
    it 'should call the before destroy method' do
      model = FixtureModel.new(:name => "test model")
      model.should_receive(:before_destroy_test_blob).should be_true
      model.save
      model.destroy
    end
  end
  
  describe "add a blob entity to the model" do
    it 'should have a blob_test_id attribute' do
      dataset = "\1" * 10
      
      model = FixtureModel.new(:name => "test model")
      model.test_blob=dataset
      model.save.should be_true
      model.test_blob_id.should_not be_nil
      
    end
    it 'should have a blob relation' do
      dataset = "\1" * 10
      
      model = FixtureModel.new(:name=> "test model")
      model.test_blob= dataset
      model.save
      model.test_blob.should_not be_nil
    end
  end
  
  describe "update a blob entity to the model" do
    it 'should update a blob_test_id attribute' do
      dataset = "\1" * 10
      
      model = FixtureModel.new(:name => "test model")
      model.test_blob= dataset
      model.save
      
      update_dataset = "\1" * 100
      model.test_blob= update_dataset
      #TODO: update without before  :save???
      model.save
      
      model.test_blob.size.should == 100
    end
    
  end
  
  describe "destroy a blob entity with the model" do
    it 'should not have a blob_entity with the given key' do
      dataset = "\1" * 10
      
      model = FixtureModel.new(:name => "test model")
      model.test_blob= dataset
      model.save
      
      key = model.test_blob_id
      
      model.destroy
      
      @entity = AppEngineUtils::Datastore::Blob.new
      @entity.read(key).should be_nil
    end
  end
  
  describe "read a blob entity with the model" do
    it 'should read a blob_entity with the given key' do
      dataset = "\1" * 10
      
      model = FixtureModel.new(:name => "test model last")
      model.test_blob= dataset
      model.save
      read_model = FixtureModel.first(:name => "test model last")
      read_model.test_blob.should_not be_nil
    end
  end
end


class FixtureModel 
  include DataMapper::Resource
  
  property :id, Serial
  property :name, String
  
  blob :test_blob
end
